package discord

import (
	dsmodel "dorisbot/models/discord"
	pubmodel "dorisbot/models/public"
	"errors"
	"log"
)

// Errors
var (
	ErrCheckServer    = errors.New("check server info fail")
	ErrServerExists   = errors.New("server already exists")
	ErrCreateServer   = errors.New("create server fail")
	ErrGetServerOwner = errors.New("get server owner fail")
	ErrWriteServer    = errors.New("write server data fail")
)

func addServer(info EvtInfo, name string) string {
	exists, err := dsmodel.GetServerByID(info.Server)
	if err != nil {
		log.Println("check server exists fail :: ", err)
		return ErrCheckServer.Error()
	}
	if exists != nil {
		return ErrServerExists.Error()
	}

	ds, err := dsmodel.NewServer(info.Server, name, 0)
	if err != nil {
		log.Println("create dsmodel fail :: ", err)
		return "create server data fail"
	}

	log.Println("show info :: ", info)
	owner, err := client.GetGuildOwner(info.Server)
	if err != nil {
		log.Println("get server owner err :: ", err)
		return "get server owner fail"
	}
	ds.Owner = owner

	err = ds.Write()
	if err != nil {
		log.Println("write server data fail :: ", err)
		return "write server data fail"
	}

	return "add server success"
}

func addChannel(info EvtInfo, cmd, notify string) string {
	server, err := dsmodel.GetServerByID(info.Server)
	if err != nil {
		return "Get server data fail"
	}
	_ = server

	if server == nil {
		return "Server not init"
	}

	ch, err := dsmodel.NewChannel(info.Channel, info.Server, cmd == "1", notify == "1")
	if err != nil {
		return "init channel data fail"
	}
	err = ch.Write()
	if err != nil {
		return "write channel data fail"
	}

	return "Success"
}

func addFacebookPage(info EvtInfo, fbid, tmpl string) string {
	log.Printf("add facebook ::: id : %s  ::: tmpl : %s \n", fbid, tmpl)
	ch, err := dsmodel.GetChannelByID(info.Channel)
	if err != nil {
		log.Println("get channel fail", err)
		return ""
	}

	var page *pubmodel.FacebookPageModel

	page, err = pubmodel.GetFacebookPageByID(fbid)
	if err != nil {
		return "check facebook fail"
	}
	if page == nil {
		page, err = pubmodel.NewFacebookPage(fbid)
		if err != nil {
			return "write data fail"
		}
		err = page.Write()
		if err != nil {
			return "write data fail"
		}
		rt, err := dsmodel.CreateFacebookRT(info.Channel, page.ID, tmpl)
		if err != nil {
			log.Println("create fail :: ", err)
			return "create data fail"
		}

		err = rt.Write()
		if err != nil {
			return "write data fail"
		}
	} else {
		fbrt, err := ch.GetFacebookPageByID(fbid)
		if err != nil {
			log.Println("get fb rt fail :: ", err)
			return "check data fail"
		}

		if fbrt != nil {
			return "facebook page already linked"
		}

		rt, err := dsmodel.CreateFacebookRT(info.Channel, page.ID, tmpl)
		if err != nil {
			log.Println("create fail :: ", err)
			return "create data fail"
		}

		err = rt.Write()
		if err != nil {
			return "write data fail"
		}
	}
	return "Success"
}

func delFacebookPage(info EvtInfo, fbid string) string {
	ch, err := dsmodel.GetChannelByID(info.Channel)
	if err != nil {
		return ""
	}

	err = ch.DeleteFacebookPage(fbid)
	if err != nil {
		return "delete facebook page fail"
	}

	return "Success"
}

func addInstagram(info EvtInfo, igid, tmpl string) string {
	log.Printf("add instagram :: ID: %v , tmpl: %v\n", igid, tmpl)
	if len(igid) == 0 || len(tmpl) == 0 {
		return "id or tmpl is empty"
	}
	ch, err := dsmodel.GetChannelByID(info.Channel)
	if err != nil {
		return "get channel data fail"
	}
	ig, err := pubmodel.GetInstagramByID(igid)
	if err != nil {
		return "get instagram data fail"
	}
	if ig == nil {
		ig, err = pubmodel.NewInstagram(igid)
		if err != nil {
			return "create instagram fail"
		}
		err = ig.Write()
		if err != nil {
			return "write instagram data fail"
		}

		rt, err := dsmodel.CreateInstagramRT(ch.ID, igid, tmpl)
		if err != nil {
			return "create channel instagram link fail"
		}
		if err = rt.Write(); err != nil {
			return "write instagram link data fail"
		}
	} else {
		igrt, err := ch.GetInstagramByID(igid)
		if err != nil {
			return "get instagram rt fail"
		}
		if igrt != nil {
			return "instagram already linked"
		}

		rt, err := dsmodel.CreateInstagramRT(ch.ID, igid, tmpl)
		if err != nil {
			return "create channel instagram link fail"
		}
		if err := rt.Write(); err != nil {
			return "write instagram link data fail"
		}
	}
	return "Success"
}

func delInstagram(info EvtInfo, igid string) string {
	ch, err := dsmodel.GetChannelByID(info.Channel)
	if err != nil {
		return "get channel data fail"
	}
	err = ch.DeleteInstagram(igid)
	if err != nil {
		return "delete instagram data fail"
	}

	return "Success"
}
